/*
 * $Id: cmd_quit.c 1079 2004-10-22 19:59:34Z troyhebe $
 *
 * This file is part of lcrash, an analysis tool for Linux memory dumps.
 *
 * Created by Silicon Graphics, Inc.
 * Contributions by IBM, and others
 *
 * Copyright (C) 1999 - 2002 Silicon Graphics, Inc. All rights reserved.
 * Copyright (C) 2001, 2002 IBM Deutschland Entwicklung GmbH, IBM Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#include <lcrash.h>

/*
 * quit_cmd() -- Run the 'quit' command.
 */
int
quit_cmd(command_t *cmd)
{
	char str[256];

	if (!strcmp(cmd->command, "q")) {
		fprintf(cmd->ofp, "Do you really want to quit (y to quit) ? ");
		get_string(str, 256);
		if (str[0] == 'y' || str[0] == 'Y') {
			/* FIXME: free trees??? */
			exit(0);
		}
	}
	else if (!strcmp(cmd->command, "quit") || !strcmp(cmd->command, "q!")) {
		/* FIXME: free trees??? */
		exit(0);
	}
	return 1;
}

#define _QUIT_USAGE ""

/*
 * quit_usage() -- Print the usage string for the 'quit' command.
 */
void
quit_usage(command_t *cmd)
{
	CMD_USAGE(cmd, _QUIT_USAGE);
}

/*
 * quit_help() -- Print the help information for the 'quit' command.
 */
void
quit_help(command_t *cmd)
{
	CMD_HELP(cmd, _QUIT_USAGE,
	"Exit lcrash.  Note that q will prompt for confirmation unless a "
	"'!' is appended to the command line.");
}

/*
 * quit_parse() -- Parse the command line arguments for 'quit'.
 */
int
quit_parse(command_t *cmd)
{
	if (set_cmd_flags(cmd, C_FALSE, 0)) {
		return(1);
	}
	return(0);
}
