/*
 * $Id: kl_dump_ia64.c 1079 2004-10-22 19:59:34Z troyhebe $
 *
 * This file is part of libklib.
 * A library which provides access to Linux system kernel dumps.
 *
 * Created by Silicon Graphics, Inc.
 * Contributions by IBM, NEC, and others
 *
 * Copyright (C) 1999 - 2002, 2004 Silicon Graphics, Inc. All rights reserved.
 * Copyright (C) 2001, 2002 IBM Deutschland Entwicklung GmbH, IBM Corporation
 * Copyright 2000 Junichi Nomura, NEC Solutions <j-nomura@ce.jp.nec.com>
 *
 * This code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#include <klib.h>
#include <linux/stddef.h>	/*offsetof*/

extern off_t dump_header_offset;
extern void *G_dump_header;
extern void *G_dump_header_asm;

int kl_header_size(generic_dump_header_t *);
uint64_t kl_header_magic(generic_dump_header_t *);
void *kl_read_header(int fd, generic_dump_header_t *);

static int
_get_dump_header_asm(int fd)
{
	int header_size, header_asm_size;
	uint64_t magic;
	generic_dump_header_t gdhp;

	header_size = kl_header_size(G_dump_header);

	/* Step over the dump header and read the dump_header_asm 
	 * in from disk...
         */
        if (lseek(fd, dump_header_offset+header_size, SEEK_SET) < 0) {
                KL_ERROR = KLE_INVALID_LSEEK;
                return(1);
        }

        /* Now read in the generic dump_header for the arch specific        
         * dump_header and make sure it's valid. 
         */
        if ((read(fd, &gdhp, sizeof(gdhp)) != sizeof(gdhp))) {
                KL_ERROR = KLE_INVALID_DUMP_HEADER;
                return(1);
        }

	/* Verify the magic number for this dump_header...
	 */
	magic = kl_header_magic(&gdhp);
	if (magic != KL_DUMP_MAGIC_NUMBER_IA64) {
		KL_ERROR = KLE_INVALID_DUMP_HEADER;
                return(1);
	}
	header_asm_size = kl_header_size(&gdhp);
	
        /* Rewind and read in the asm dump_header.
         */
        if (lseek(fd, dump_header_offset+header_size, SEEK_SET) < 0) {
                KL_ERROR = KLE_INVALID_LSEEK;
                return(1);
        }

	G_dump_header_asm = kl_read_header(fd, &gdhp);

	return(0);
}

/*
 * kl_get_dump_header_ia64()
 */
int
kl_get_dump_header_ia64(void)
{
	/* first, make sure this isn't a live system
	 */
	if (MIP->core_type != reg_core) {
		KL_ERROR = KLE_ACTIVE;
		return(1);
	}
	(void)_get_dump_header_asm(MIP->core_fd);
	return(0);

}

int
kl_set_dumparch_ia64(void)
{
	KL_PAGE_OFFSET  = KL_PAGE_OFFSET_IA64;
	KL_STACK_OFFSET = KL_KSTACK_SIZE_IA64;
	KL_PAGE_SHIFT   = KL_PAGE_SHIFT_IA64;
	KL_PAGE_SIZE    = KL_PAGE_SIZE_IA64;
	KL_PAGE_MASK    = KL_PAGE_MASK_IA64;
	KLP->dump->arch.pgdshift = KL_PGDIR_SHIFT_IA64;
	KLP->dump->arch.pgdsize = KL_PGDIR_SIZE_IA64;
	KLP->dump->arch.pgdmask = KL_PGDIR_MASK_IA64;
	KLP->dump->arch.pmdshift = KL_PMD_SHIFT_IA64;
	KLP->dump->arch.pmdsize = KL_PMD_SIZE_IA64;
	KLP->dump->arch.pmdmask = KL_PMD_MASK_IA64;
	KLP->dump->arch.ptrsperpgd = KL_PTRS_PER_PGD_IA64;
	KLP->dump->arch.ptrsperpmd = KL_PTRS_PER_PMD_IA64;
	KLP->dump->arch.ptrsperpte = KL_PTRS_PER_PTE_IA64;
	KLP->dump->arch.kernelstack = kl_kernelstack_ia64;
	KLP->dump->arch.virtop = kl_virtop_ia64;
	KLP->dump->arch.mmap_virtop = kl_mmap_virtop_ia64;
	KLP->dump->arch.init_virtop = kl_init_virtop_ia64;
	KLP->dump->arch.valid_physmem = kl_valid_physmem_ia64;	
	KLP->dump->arch.next_valid_physaddr = kl_next_valid_physaddr_ia64;
	KLP->dump->arch.fix_vaddr = kl_fix_vaddr_ia64;

	return(0);
}
