/*
 * lkcd_config.c -- dump configuration utility
 *
 * Created by: Matt D. Robinson (yakker@aparity.com)
 * Network dump configuration support by Bharata B Rao (bharata@in.ibm.com)
 *
 * Copyright 2001 Matt D. Robinson (yakker@aparity.com), all rights reserved.
 * Copyright (C) 2002 International Business Machines Corp. 
 *
 * This source code is released under the terms of version 2 of the GNU GPL.
 */
/* #include <lcrash.h> */
#include <klib.h>

#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <netinet/ether.h>

#define DUMP_DEVICE	"/dev/dump"
#define DUMP_TRUE	1
#define DUMP_FALSE	0

/*
 * Name: main()
 * Func: A quick-and-dirty dump configuration tool.  Should suffice
 *       for the time being.
 */
int
main(int argc, char **argv)
{
	dev_t dnum = 0;
	int dfd, fd, err, device_num, c;
	unsigned long target_port, source_port, flags, level;
	unsigned long compress, dumpokay;
	int device_name_set = DUMP_FALSE, level_set = DUMP_FALSE;
	int flags_set = DUMP_FALSE, compress_set = DUMP_FALSE;
	int device_num_set = DUMP_FALSE, query_set = DUMP_FALSE;
	int manual_dump_set = DUMP_FALSE;
	int target_host_set = DUMP_FALSE, target_port_set = DUMP_FALSE;
	int source_port_set = DUMP_FALSE, eth_addr_set = DUMP_FALSE;
	char *device_name, tbuf[1024], *target_host, *eth_addr;
	struct stat stbuf;

#if !DUMP_DEBUG
	/* check for root if not debugging */
	if (geteuid() != 0) {
		fprintf(stderr, "Error: You must run this program as root!\n");
		return (-1);
	}
#endif

	/* walk through the options */
	compress = level = flags = device_num = 0;
	while ((c = getopt(argc, argv, "qc:d:D:f:l:t:p:P:e:m")) > 0) {
		switch (c) {
			case 'c':
				if (compress_set == DUMP_TRUE) {
					goto usage;
				}
				compress_set = DUMP_TRUE;
				compress = atoi(optarg);
				break;

			case 'd':
				if ((device_name_set == DUMP_TRUE) ||
					(device_num_set == DUMP_TRUE)) {
						goto usage;
				}
				device_name_set = DUMP_TRUE;
				device_name = optarg;
				break;

			case 'D':
				if ((device_name_set == DUMP_TRUE) ||
					(device_num_set == DUMP_TRUE)) {
						goto usage;
				}
				device_num_set = DUMP_TRUE;
				device_num = atoi(optarg);
				break;

			case 'f':
				if (flags_set == DUMP_TRUE) {
					goto usage;
				}
				flags_set = DUMP_TRUE;
				flags = strtoul(optarg, (char **)NULL, 16);
				break;

			case 'l':
				if (level_set == DUMP_TRUE) {
					goto usage;
				}
				level_set = DUMP_TRUE;
				level = atoi(optarg);
				break;

			case 'm':
				if (manual_dump_set == DUMP_TRUE) {
					goto usage;
				}
				manual_dump_set = DUMP_TRUE;
				break;

			case 'q':
				if (query_set == DUMP_TRUE) {
					goto usage;
				}
				query_set = DUMP_TRUE;
				break;

			case 't':
				/* target host */
				if (target_host_set == DUMP_TRUE) {
					goto usage;
				}
				target_host_set = DUMP_TRUE;
				target_host = optarg;
				break;

			case 'p':
				/* source port */
				if (source_port_set == DUMP_TRUE) {
					goto usage;
				}
				source_port_set = DUMP_TRUE;
				source_port = atoi(optarg);
				break;

			case 'P':
				/* target port */
				if (target_port_set == DUMP_TRUE) {
					goto usage;
				}
				target_port_set = DUMP_TRUE;
				target_port = atoi(optarg);
				break;

			case 'e':
				/* ethernet address */
				if (eth_addr_set == DUMP_TRUE) {
					goto usage;
				}
				eth_addr_set = DUMP_TRUE;
				eth_addr = optarg;
				break;

			default:
				goto usage;
		}
	}

	/* make sure -q isn't specified with anything else */
	if (query_set == DUMP_TRUE) {
		if ((device_name_set == DUMP_TRUE) ||
			(device_num_set == DUMP_TRUE) ||
			(level_set == DUMP_TRUE) ||
			(flags_set == DUMP_TRUE) ||
			(compress_set == DUMP_TRUE)) {
				goto usage;
		}

		/* open the dump device for queries */
		if ((dfd = open(DUMP_DEVICE, O_RDONLY)) < 0) {
			perror("open of dump device");
			return (dfd);
		}
	
		/* check if LKCD has been configured */
		if ((err = ioctl(dfd,DIOGDUMPOKAY, (caddr_t)&dumpokay)) < 0) {
			perror("ioctl() query for dump configuration status failed");
			close(dfd);
			return (err);
		}

		/* Do not proceed if dump hasn't been configured */
		if(!dumpokay) {
			printf("LKCD not yet configured. Run lkcd config\n");
			close(dfd);
			return(0);
		}

		/* get dump compression */
		if ((err = ioctl(dfd, DIOGDUMPCOMPRESS, (caddr_t)&compress)) < 0) {
			perror("ioctl() query for dump compression failed");
			close(dfd);
			return (err);
		}

		/* get dump flags */
		if ((err = ioctl(dfd, DIOGDUMPFLAGS, (caddr_t)&flags)) < 0) {
			perror("ioctl() query for dump flags failed");
			close(dfd);
			return (err);
		}

		/* get dump level */
		if ((err = ioctl(dfd, DIOGDUMPLEVEL, (caddr_t)&level)) < 0) {
			perror("ioctl() query for dump level failed");
			close(dfd);
			return (err);
		}

		/* get device to dump to (if specified) */
		if ((err = ioctl(dfd, DIOGDUMPDEV, (caddr_t)&dnum)) < 0) {
			perror("ioctl() for dump device failed");
			close(dfd);
			return (err);
		}

		printf("            Configured dump device: 0x%x\n", (int)dnum);

		memset(tbuf, 0, 1024);
		if (flags == KL_DUMP_FLAGS_NONE) {
			strcat(tbuf, "KL_DUMP_FLAGS_NONE|");
		} else if (flags & KL_DUMP_FLAGS_NONDISRUPT) {
			strcat(tbuf, "KL_DUMP_FLAGS_NONDISRUPT|");
		} else if (flags & KL_DUMP_FLAGS_NETDUMP) {
			strcat(tbuf, "KL_DUMP_FLAGS_NETDUMP|");
		} else if (flags & KL_DUMP_FLAGS_DISKDUMP) {
			strcat(tbuf, "KL_DUMP_FLAGS_DISKDUMP|");
		}
		if (tbuf[strlen(tbuf)-1] == '|') {
			tbuf[strlen(tbuf)-1] = 0;
		}
		printf("             Configured dump flags: %s\n", tbuf);

		memset(tbuf, 0, 1024);
		if (level == KL_DUMP_LEVEL_NONE) {
			strcat(tbuf, "KL_DUMP_LEVEL_NONE|");
		}
		if (level & KL_DUMP_LEVEL_HEADER) {
			strcat(tbuf, "KL_DUMP_LEVEL_HEADER|");
		}
		if (level & KL_DUMP_LEVEL_KERN) {
			strcat(tbuf, "KL_DUMP_LEVEL_KERN|");
		}
		if (level & KL_DUMP_LEVEL_USED) {
			strcat(tbuf, "KL_DUMP_LEVEL_USED|");
		}
		if (level & KL_DUMP_LEVEL_ALL_RAM) {
			strcat(tbuf, "KL_DUMP_LEVEL_ALL_RAM|");
		}
		if (level & KL_DUMP_LEVEL_ALL) {
			strcat(tbuf, "KL_DUMP_LEVEL_ALL|");
		}
		if (tbuf[strlen(tbuf)-1] == '|') {
			tbuf[strlen(tbuf)-1] = 0;
		}
		printf("             Configured dump level: %s\n", tbuf);

		memset(tbuf, 0, 1024);
		if (compress == KL_DUMP_COMPRESS_NONE) {
			strcat(tbuf, "KL_DUMP_COMPRESS_NONE|");
		}
		if (compress & KL_DUMP_COMPRESS_RLE) {
			strcat(tbuf, "KL_DUMP_COMPRESS_RLE|");
		}
		if (compress & KL_DUMP_COMPRESS_GZIP) {
			strcat(tbuf, "KL_DUMP_COMPRESS_GZIP|");
		}
		if (tbuf[strlen(tbuf)-1] == '|') {
			tbuf[strlen(tbuf)-1] = 0;
		}
		printf("Configured dump compression method: %s\n", tbuf);

		/* bail out */
		close(dfd);
		return (0);

	} else if (manual_dump_set == DUMP_TRUE) {
		if ((device_name_set == DUMP_TRUE) ||
			(device_num_set == DUMP_TRUE) ||
			(level_set == DUMP_TRUE) ||
			(flags_set == DUMP_TRUE) ||
			(compress_set == DUMP_TRUE)) {
				goto usage;
		}

		/* open the dump device - dummy open */
		if ((dfd = open(DUMP_DEVICE, O_RDONLY)) < 0) {
			perror("open of dump device");
			return (dfd);
		}
	
		/* Take the dump, dumpokay is not used */
		if ((err = ioctl(dfd,DIOSDUMPTAKE, (caddr_t)(long)dumpokay)) < 0) {
			printf("ioctl() query for manual dump failed\n");
			if(errno == ENODEV)
			printf("LKCD not configured. Run lkcd config\n");
			close(dfd);
			return (err);
		}
	 
		/* bail out */
		close(dfd);
		return (0);

	} else if ((device_name_set != DUMP_TRUE) && (device_num_set != DUMP_TRUE) && 
		   (level_set != DUMP_TRUE) && (flags_set != DUMP_TRUE) && 
		   (compress_set != DUMP_TRUE)) {
			/* make sure _something_ is set */
			goto usage;
	}


	/* open the dump device for setting dump fields */
	if ((dfd = open(DUMP_DEVICE, O_RDWR)) < 0) {
		perror("open of dump device");
		return (dfd);
	}

	/* set dump flags */
	if (flags_set == DUMP_TRUE) {
		if (!flags) {
			printf("dump flags not specified\n");
			return (-1);
		}

		/* check if dump target is specified */
		if (!(flags & KL_DUMP_FLAGS_DISKDUMP) &&
				!(flags & KL_DUMP_FLAGS_NETDUMP)) {
			printf("dump target not specified in dump flags\n");
			return (-1);
		}

		/* One and only one dump target type should be specified */
		if ((flags & KL_DUMP_FLAGS_DISKDUMP) &&
				(flags & KL_DUMP_FLAGS_NETDUMP)) {
			printf("only one dump target can be specified\n");
			return (-1);
		}

		if ((err = ioctl(dfd, DIOSDUMPFLAGS,
				 (caddr_t)(long)flags)) < 0) {
			perror("ioctl() for dump flags failed");
			close(dfd);
			return (err);
		}
	}

	/* set the dnum -- dnum will != 0 after */
	if (flags_set && (flags & KL_DUMP_FLAGS_DISKDUMP)) {
		if (device_name_set == DUMP_TRUE) {
			if ((fd = open(device_name, O_RDONLY)) < 0) {
				perror("open of device name");
				close(dfd);
				return (fd);
			}

			if ((err = fstat(fd, &stbuf)) < 0) {
				perror("fstat of device name");
				close(dfd);
				close(fd);
				return (err);
			}
			dnum = stbuf.st_rdev;
			close(fd);
		} else if (device_num_set == DUMP_TRUE) {
			if (device_num <= 0) {
				fprintf(stderr, "Error: -d value must be > 0!\n");
				close(dfd);
				return (-1);
			}
			dnum = (dev_t)device_num;
		}
	}

	/* set dump level */
	if (level_set == DUMP_TRUE) {
		if ((err = ioctl(dfd, DIOSDUMPLEVEL,
				 (caddr_t)(long)level)) < 0) {
			perror("ioctl() for dump level failed");
			close(dfd);
			return (err);
		}
	}
	
	/* configure network dump parameters */
	if (flags_set && (flags & KL_DUMP_FLAGS_NETDUMP)) {
		/* set target ip */
		if (target_host_set) {
			struct hostent *host = gethostbyname(target_host);
			if (!host) {
				fprintf(stderr, "gethostbyname: %s\n",
						hstrerror(h_errno));
				return (-1);
			}
			if ((err = ioctl(dfd, DIOSTARGETIP,
			(unsigned long)*(unsigned int *)*host->h_addr_list)) < 0) {
				perror("ioctl() for target ip failed");
				close(dfd);
				return (err);
			}
		}
			
		/* set target port */
		if (target_port_set) {
			if ((err = ioctl(dfd, DIOSTARGETPORT,
					(caddr_t)target_port)) < 0) {
				perror("ioctl() for target port failed");
				close(dfd);
				return (err);
			}
		}

		/* set source port */
		if (source_port_set) {
			if ((err = ioctl(dfd, DIOSSOURCEPORT,
					(caddr_t)source_port)) < 0) {
				perror("ioctl() for source port failed");
				close(dfd);
				return (err);
			}
		}

		/* set ethernet address */
		if (eth_addr_set) {
			struct ether_addr *ether;
			if (!(ether = ether_aton(eth_addr))) {
				fprintf(stderr, "Invalid ethernet address %s\n",
						eth_addr);
				return (-1);
			}
			if ((err = ioctl(dfd, DIOSETHADDR,
					(caddr_t)ether)) < 0) {
				perror("ioctl() for ethernet addr failed");
				close(dfd);
				return (err);
			}
		}
	}

	/*
	 * set device to dump to (if specified)  - this should be done
	 * last as network dump attempts a handshake during this ioctl.
	 * All network dump parameters need to be set properly before
	 * handshaking.
	 */
	if (flags_set && (flags & KL_DUMP_FLAGS_NETDUMP)) {
		/* network device */
		/*
		 * check if all parameters needed by network dump are
		 * specified
		 */
		if (!target_host_set || !source_port_set || !target_port_set
				|| !eth_addr_set) {
			printf("Not all configuration items needed by network "
				"dump specified in the configuration file");
			return (-1);
		}

		/*
		 * presently device name is passed to the kernel.
		 * can get the interface index by using socket ioctl
		 * and it can be passed using dnum only - necessary ?
		 */
		if (device_name_set && device_name) {
			if ((err = ioctl(dfd, DIOSDUMPDEV,
					(caddr_t)device_name)) < 0) {
				perror("ioctl() for network device failed");
				close(dfd);
				return (err);
			}
		}
	} else {
		/* block device */
		if (dnum != (dev_t)0) {
			if ((err = ioctl(dfd, DIOSDUMPDEV, (caddr_t)(long)dnum)) < 0) {
				perror("ioctl() for dump device failed");
				close(dfd);
				return (err);
			}
		}
	}

	/* set dump compression */
	if (compress_set == DUMP_TRUE) {
		if ((err = ioctl(dfd, DIOSDUMPCOMPRESS,
				 (caddr_t)(long)compress)) < 0) {
			perror("ioctl() for dump compression failed");
			close(dfd);
			return (err);
		}
	}

	return (0);

usage:
	printf("Usage: %s [-d device_name | -D device_num] "
		"[-c compression_type]\n\t\t\t[-f flags] [-l level]\n"
#if DUMP_DEBUG
		"\t\t\t[-t (panic || die || dump || forever) ]\n"
#endif
		"\nUsage: %s [-q]\n", argv[0], argv[0]);
	return (1);
}
