#ifndef NETDUMPCLIENT_H
#define NETDUMPCLIENT_H

#include <glib.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>

typedef struct _NetdumpClient NetdumpClient;


typedef struct {
  guint32 page_nr;
  char   *data;
  guint32 sections_recieved;
  guint32 request_nr;
  guint   n_timeouts;
  guint   timeout_tag;
} OutstandingPage;

struct _NetdumpClient {
  guint32 ip;
  gboolean just_reboot;
  
  guint32 outstanding_request;
  guint   timeout_tag;
  guint   n_timeouts;
  guint   n_handshakes;

  unsigned char magic_value[8];
  
  /* Host data */
  guint32 nr_pages;
  guint32 page_size;
  guint32 page_offset;

  gchar  *regs;
  guint   regs_len;
  
  guint32 current_page;
  OutstandingPage *outstanding;
  int     n_outstanding;
  
  void (*timeout)        (NetdumpClient *client,
			  guint32        request_nr);
  void (*process_packet) (NetdumpClient *client,
			  struct sockaddr_in *addr,
			  unsigned char *buffer,
			  int buf_len);


  /* Coredump data: */
  int    dump_fd;
  off_t data_offset;

  char *dump_dir;
};


NetdumpClient *netdump_client_new     (guint32        ip,
				       const char    *dir,
				       gboolean       just_reboot);
void           netdump_client_destroy (NetdumpClient *client);
guint          netdump_client_hash    (gconstpointer  key);
gint           netdump_client_compare (gconstpointer  a,
				       gconstpointer  b);


#endif
