/*-*-c++-*-
 * $Id: cconfigmanager.h 1079 2004-10-22 19:59:34Z troyhebe $
 *
 * This file is part of qlcrash, a GUI for the dump-analysis tool lcrash.
 *
 * Copyright (C) 2001 IBM Deutschland Entwicklung GmbH, IBM Corporation
 *
 * Authors:
 * Michael Geselbracht (let@users.sourceforge.net)
 * Fritz Elfert (elfert@de.ibm.com)
 * Michael Holzheu (holzheu@de.ibm.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */
#ifndef CCONFIGMANAGER_H
#define CCONFIGMANAGER_H

#ifndef WIN32
# ifndef _GNU_SOURCE
#  define _GNU_SOURCE
# endif
#endif

#include <qobject.h>
#include <qstring.h>
#include "osdep.h"
#include <map> // STL

// On Windows, we use the a hierarchy below
// the following registry path:
#ifdef WIN32
#define MY_KEYROOT HKEY_CURRENT_USER
#define MY_KEYHOME "Software\\GNU\\QLcrash"
#endif

using namespace std;

/* Configure groups */
#define CFG_GRP_CONSOLE                "Console"
#define CFG_GRP_FILES                   "Files"
#define CFG_GRP_GRAPH                   "Graph"
#define CFG_GRP_DUMP                    "Dump"
#define CFG_GRP_DISASSEMBLE             "Disassemble"
#define CFG_GRP_TOOLBAR                 "Toolbar"
#define CFG_GRP_APP                     "Application"
#define CFG_GRP_LAYOUT                  "Layout"
#define CFG_GRP_PLUGIN                  "Plugin"

/* Configure attributes */
#define CFG_ATT_CONSOLE_FONTFAMILY      "fontfamily"
#define CFG_ATT_CONSOLE_FONTSIZE        "fontsize"
#define CFG_ATT_FONTFAMILY              "fontfamily"
#define CFG_ATT_FONTSIZE                "fontsize"
#define CFG_ATT_USEFIXEDFONT            "use_fixed_font"
#define CFG_ATT_FILES_LCRASH            "lcrash"
#define CFG_ATT_FILES_LCMAPFILE         "mapfile"
#define CFG_ATT_FILES_LCDUMP            "dumpfile"
#define CFG_ATT_FILES_LCKERNTYPES       "kerntypes"
#define CFG_ATT_GRAPH_PRINTFORMAT       "printformat"
#define CFG_ATT_GRAPH_ACCELERATION      "accerleration"
#define CFG_ATT_DUMP_TBDOCKMAIN         "toolbar_dockmain"
#define CFG_ATT_TOOLBAR_MAIN            "mainbar"
#define CFG_ATT_TOOLBAR_MAINEDGE        "main_dock"
#define CFG_ATT_TOOLBAR_MAINNL          "main_nl"
#define CFG_ATT_TOOLBAR_MAININDEX       "main_index"
#define CFG_ATT_TOOLBAR_MAINEXTRA       "main_extra_offset"
#define CFG_ATT_TOOLBAR_STATUS          "statusbar"
#define CFG_ATT_TOOLBAR_CHILD           "childbar"
#define CFG_ATT_TOOLBAR_CHILDEDGE       "child_dock"
#define CFG_ATT_TOOLBAR_CHILDNL         "child_nl"
#define CFG_ATT_TOOLBAR_CHILDINDEX      "child_index"
#define CFG_ATT_TOOLBAR_CHILDEXTRA      "child_extra_offset"
#define CFG_ATT_APP_FONTFAMILY          "fontfamily"
#define CFG_ATT_APP_FONTSIZE            "fontsize"
#define CFG_ATT_APP_FIXEDFONTFAMILY     "fixedfontfamily"
#define CFG_ATT_APP_FIXEDFONTSIZE       "fixedfontsize"
#define CFG_ATT_APP_SAVEWINDOWSIZE      "save_window_size"
#define CFG_ATT_APP_WINDOWWIDTH         "window_width"
#define CFG_ATT_APP_WINDOWHEIGHT        "window_height"
#define CFG_ATT_APP_WINDOWMAXIMIZED     "window_maximized"
#define CFG_ATT_APP_WIDGETSTYLE         "widget_style"
#define CFG_ATT_APP_HIGHLIGHTTASKS      "highlight_tasks"
#define CFG_ATT_APP_CONFIRMEXIT         "confirm_exit"
#define CFG_ATT_LAYOUT_GRIDX            "grid_x"
#define CFG_ATT_LAYOUT_GRIDY            "grid_y"
#define CFG_ATT_LAYOUT_SIZE             "size"
#define CFG_ATT_LAYOUT_AUTOINSERT       "auto_insert"
#define CFG_ATT_LAYOUT_AUTOREMOVE       "auto_remove"
#define CFG_ATT_LAYOUT_AUTOEDGE         "auto_edge"
#define CFG_ATT_LAYOUT_ALIAS            "aliasdetection"
#define CFG_ATT_FILES_PLUGIN            "currentplugin"

/* Default values */
#define DEF_LAYOUT_GRIDX                150
#define DEF_LAYOUT_GRIDY                30
#define DEF_LAYOUT_SIZE                 16000
#define DEF_LAYOUT_AUTOINSERT           0
#define DEF_LAYOUT_AUTOREMOVE           0
#define DEF_LAYOUT_AUTOEDGE             0
#define DEF_LAYOUT_ALIAS                1
#ifdef WIN32
#  define DEF_FILES_PLUGIN                "tcpplugin.dll"
#else
#  define DEF_FILES_PLUGIN                "localplugin.so"
#endif
#define DEF_FILES_LCRASH                "/sbin/lcrash"
#define DEF_FILES_LCMAPFILE             "/boot/System.map"
#define DEF_FILES_LCDUMP                "/dev/mem"
#define DEF_FILES_LCKERNTYPES           "/boot/kerntypes"
#define DEF_GRAPH_PRINTFORMAT           0
#define DEF_GRAPH_ACCELERATION          1
#define DEF_DUMP_TBDOCKMAIN             0
#ifdef WIN32
#  define DEF_APP_FONTFAMILY              "Arial"
#  define DEF_APP_FIXEDFONTFAMILY         "Courier"
#  define DEF_DISASSEMBLE_FONTFAMILY      "Courier"
#  define DEF_DUMP_FONTFAMILY             "Courier"
#  define DEF_APP_FONTSIZE                8
#  define DEF_APP_FIXEDFONTSIZE           8
#  define DEF_DISASSEMBLE_FONTSIZE        8
#  define DEF_DUMP_FONTSIZE               8
#else
#  define DEF_APP_FONTFAMILY              "Helvetica"
#  define DEF_APP_FIXEDFONTFAMILY         "Courier"
#  define DEF_DISASSEMBLE_FONTFAMILY      "Courier"
#  define DEF_DUMP_FONTFAMILY             "Courier"
#  define DEF_APP_FONTSIZE                12
#  define DEF_APP_FIXEDFONTSIZE           12
#  define DEF_DISASSEMBLE_FONTSIZE        12
#  define DEF_DUMP_FONTSIZE               12
#endif
#define DEF_APP_WIDGETSTYLE             0
#define DEF_APP_HIGHLIGHTTASKS          0
#define DEF_APP_SAVEWINDOWSIZE          1
#define DEF_APP_CONFIRMEXIT             1
#define DEF_DUMP_TBDOCKMAIN             0
#define DEF_USEFIXEDFONT                1

/**
  * @author Michael Geselbracht
  */

class CConfigManager : QObject {
public:
	CConfigManager(QObject* parent, const char* name=0);
	~CConfigManager();
	
	/* set the name of the configuration file */
	bool setFile(const QString& fname);
	
	/* save current configuration to config file */
	bool saveConfig();
	
	inline QString file() const {
		return oFile;
	}
	
	void setItem(const QString& category, const QString& name, const QString& value);
	void setItem(const QString& category, const QString& name, int value);
	
	QString item(const QString& category, const QString& name, const QString& def) const;
	int item(const QString& category, const QString& name, int def) const;
	
	QString homeDir() const;
	
	void clear();
	
private:
	QString oFile;
	map<QString, map<QString, QString>* > oMap;
};

#endif
