/*-*-c++-*-
 * cedge.cpp
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#include "cedge.h"
#include "cnode.h"

CEdge::CEdge()
{
}

CEdge::CEdge(const CEdge& edge)
{
	setEdge(edge);
}

CEdge::~CEdge()
{
	destroy();
}

CEdge&
CEdge::operator=(const CEdge& edge)
{
	setEdge(edge);
	
	return *this;
}

void
CEdge::setEdge(const CEdge& edge)
{
	oData = edge.oData;
}

void
CEdge::destroy()
{
}

Node
CEdge::from()
{
	return oData.from;
}

Node
CEdge::to()
{
	return oData.to;
}

void
CEdge::setFrom(Node f)
{
	oData.from = f;
}

void
CEdge::setTo(Node t)
{
	oData.to = t;
}

void
CEdge::setFromTo(Node f, Node t)
{
	setFrom(f);
	setTo(t);
}

Node
CEdge::adjacent(Node origin)
{
	return (origin == oData.from) ? oData.to : oData.from;
}

Node
CEdge::adjacentV(Node origin)
{
	Node walker;
	
	if (origin == oData.from) { // forward
		walker = oData.to;
		
		while (walker->virtualNode()) {
			walker = walker->next();
		}
	}
	else { // reverse
		walker = oData.from;
		
		while (walker->virtualNode()) {
			walker = walker->prev();
		}
	}
	
	return walker;
}

bool
CEdge::reversed() const
{
	return oData.reversed;
}

void
CEdge::setReversed(bool w)
{
	oData.reversed = w;
}

bool
CEdge::arrow() const
{
	return oData.arrow;
}

void
CEdge::setArrow(bool w)
{
	oData.arrow = w;
}

int
CEdge::value() const
{
	return oData.value;
}

QPoint
CEdge::p1() const
{
	return oData.p1;
}

void
CEdge::setP1(QPoint p)
{
	oData.p1 = p;
}

QPoint
CEdge::p2() const
{
	return oData.p2;
}

void
CEdge::setP2(QPoint p)
{
	oData.p2 = p;
}

void
CEdge::setValue(int value)
{
	oData.value = value;
}

QString
CEdge::label() const
{
	return oData.label;
}

void
CEdge::setLabel(const QString& label)
{
	oData.label = label;
}
