/*-*-c++-*-
 * cedge.h
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#ifndef CEDGE_H
#define CEDGE_H

class CGraph;
class CEdge;
class CNode;
typedef CEdge* Edge;
typedef CNode* Node;

#include <qstring.h>
#include <qpoint.h>

/**
  *@author Michael Geselbracht
  */

class CEdge {
public: 
	CEdge();
	CEdge(const CEdge&);
	~CEdge();
	
	CEdge& operator=(const CEdge&);
	
	void setEdge(const CEdge& edge);
	
	/* Nodes connected by this edge.
	*/
	
	Node from();
	Node to();
	
	// origin must be 'from' or 'to'
	Node adjacent(Node origin);
	Node adjacentV(Node origin); // skips virtual nodes
	
	bool reversed() const;
	void setReversed(bool w);
	
	/* Shall this edge be drawn with an arrow ?
	*/
	bool arrow() const;
	void setArrow(bool w);
	
	/* Edge coordinates.
	*/
	QPoint p1() const;
	void setP1(QPoint p);
	QPoint p2() const;
	void setP2(QPoint p);
	
	/* Universal edge attribute.
	*/
	int value() const;
	void setValue(int value);
	
	QString label() const;
	void setLabel(const QString& label);
	
	void setFrom(Node f);
	void setTo(Node t);
	void setFromTo(Node f, Node t);
	
private:
	void destroy();
	
private:
	struct Edge_t {
		Node from;
		Node to;
		bool reversed;
		bool arrow;
		int value;
		QString label;
		QPoint p1;
		QPoint p2;
		
		Edge_t() {
			from = to = 0;
			value = 0;
			reversed = false;
			arrow = true;
		}
	} oData;
};

#endif
