/*-*-c++-*-
 * cedgeset.h
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
*/

#ifndef CEDGESET_H
#define CEDGESET_H

#include <list>

class CGraph;
class CEdge;
class CNode;
class CEdgeSet;
typedef CEdgeSet* EdgeSet;
typedef CEdge* Edge;
typedef CNode* Node;


/**
  *@author Michael Geselbracht
  */

class CEdgeSet {
public:
	typedef std::list<Edge>::iterator Iterator;
	
	CEdgeSet();
	CEdgeSet(const CEdgeSet&);
	~CEdgeSet();
	
	CEdgeSet& operator=(const CEdgeSet&);
	
	void setEdges(const CEdgeSet& es);
	
	/* Return set of edges which start from node v.
	*/
	CEdgeSet edgesFrom(Node v);
	
	/* Return set of edges which end at node v.
	*/
	CEdgeSet edgesTo(Node v);
	
	std::list<Edge>::iterator begin();
	std::list<Edge>::iterator end();
	
	void add(Edge e);
	void add(EdgeSet es);
	void remove(Edge e);
	void clear();
	
	int count() const;
	bool empty() const;
	
private:
	void destroy();
	
private:
	struct EdgeSet_t {
		std::list<Edge> edges;
	} oEdgeSet;
};

#endif
