/***************************************************************************
                              cgraphheader.cpp
                             -------------------
    begin                : Sun Nov 11 2001
    copyright            : (C) 2001 by Michael Geselbracht
    email                : let@uni.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cgraphheader.h"

#include "viewone.xpm"
#include "viewmany.xpm"
#include "close.xpm"
#include "minimize.xpm"

#include <qlabel.h>
#include <qpixmap.h>
#include <qpushbutton.h>

CGraphHeader::CGraphHeader(QWidget* parent, const char* name) : QFrame(parent, name)
{
	initAttributes();
}

CGraphHeader::CGraphHeader(const QString& label, QWidget* parent, const char* name) : QFrame(parent, name)
{
	initAttributes();
	setLabel(label);
}

CGraphHeader::~CGraphHeader()
{
}

void
CGraphHeader::resizeEvent(QResizeEvent*)
{
	oLabel->resize(width() - oPixmap->width() - oOneMany->width() - oMinimize->width() - oClose->width(), height());
	oPixmap->resize(oPixmap->width(), height());
	oOneMany->resize(oOneMany->width(), height());
	oMinimize->resize(oMinimize->width(), height());
	oClose->resize(oClose->width(), height());
	
	oOneMany->move(oLabel->x() + oLabel->width(), 0);
	oMinimize->move(oOneMany->x() + oOneMany->width(), 0);
	oClose->move(oMinimize->x() + oMinimize->width(), 0);
}

void
CGraphHeader::setBackgroundColor(const QColor& color)
{
	oLabel->setBackgroundColor(color);
	oPixmap->setBackgroundColor(color);
}

bool
CGraphHeader::eventFilter(QObject* obj, QEvent* event)
{
	if (event->type() == QEvent::MouseButtonPress
		|| event->type() == QEvent::MouseButtonRelease
		|| event->type() == QEvent::MouseMove
	) {
		QMouseEvent* se = (QMouseEvent*) event;
		QWidget* sw = (QWidget*) obj;
		QMouseEvent e(se->type(), sw->mapToParent(se->pos()), se->globalPos(), se->button(), se->state());
		
		CGraphHeader::event(&e);
		
		return true;
	}
	
	return QFrame::eventFilter(obj, event);
}

void
CGraphHeader::setMouseTracking(bool w)
{
    oLabel->setMouseTracking(w);
    oPixmap->setMouseTracking(w);
    QFrame::setMouseTracking(w);
}

void
CGraphHeader::initAttributes()
{
	oLabel = new QLabel(this);
	oPixmap = new QLabel(this);
		
	oPixmap->resize(1, 0);
	oLabel->move(2, 1);
	oPixmap->move(1, 0);
	
	/* Create buttons.
	*/
	
	{
		QPixmap p(viewmany_xpm);
		oOneMany = new QPushButton(tr("Toggle one/many view"), this);
		oOneMany->setPixmap(p);
		oOneMany->setToggleButton(true);
		oOneMany->resize(p.width() + 4, p.height() + 4);
		
		connect(oOneMany, SIGNAL(toggled(bool)), SLOT(slotOneMany(bool)));
	}
	{
		QPixmap p(minimize_xpm);
		oMinimize = new QPushButton(tr("Minimize this object"), this);
		oMinimize->setPixmap(p);
		oMinimize->resize(p.width() + 4, p.height() + 4);
		
		connect(oMinimize, SIGNAL(clicked()), SIGNAL(sigMinimize()));
	}
	{
		QPixmap p(close_xpm);
		oClose = new QPushButton(tr("Close this object"), this);
		oClose->setPixmap(p);
		oClose->resize(p.width() + 4, p.height() + 4);
		
		connect(oClose, SIGNAL(clicked()), SIGNAL(sigClose()));
	}
	
	oLabel->installEventFilter(this);
	oPixmap->installEventFilter(this);
}

void
CGraphHeader::setLabel(const QString& label)
{
	oLabel->setText(label);
}

void
CGraphHeader::setPixmap(const QPixmap& pm)
{
	oPixmap->setPixmap(pm);
	oPixmap->resize(pm.width(), pm.height());
	resizeEvent(0);
}

void
CGraphHeader::slotOneMany(bool w)
{
	if (w) {
		QPixmap p(viewone_xpm);
		oOneMany->setPixmap(p);
	}
	else {
		QPixmap p(viewmany_xpm);
		oOneMany->setPixmap(p);
	}
}

bool
CGraphHeader::isViewMany() const
{
	return !oOneMany->isOn();
}
