/*-*-c++-*-
 * clayout.h
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#ifndef CLAYOUT_H
#define CLAYOUT_H

#include <qobject.h>
#include <qsize.h>
#include <deque>

#include "graph.h"

/**Layout base class
  *@author Michael Geselbracht
  */

class CLayout : public QObject {
	Q_OBJECT
	
public: 
	CLayout();
	CLayout(Graph G);
	~CLayout();
	
	Graph graph() const;
	void setGraph(Graph g);
	
	/* Check if there is a valid graph.
	*/
	bool checkGraph(QString=QString::null) const;
	
	/* An assigned graph will be deleted if it is no longer used by this object
	 * and `autoDelete' is `true'.
	*/
	bool autoDelete() const;
	void setAutoDelete(bool w);
	
	/* Layout all objects.
	*/
	virtual void layout() = 0;
	
	/* Layout a single object.
	*/
	virtual void layoutObject(Node) = 0;
	
	/* Do a geometric layout.
	*/
	virtual void geometricLayout(Node) = 0;
	
	/* Remove a node. The default implementation just invokes oGraph->removeNode()
	*/
	virtual void removeNode(Node);
	
	/* Remove an edge. The default implementation just invokes oGraph->removeEdge()
	*/
	virtual void removeEdge(Edge);
	
	/* Make graph acyclic by reversing some edges. These edges are marked as `reversed'.
	 * Note that the `value' attribute of each node will be altered.
	*/
	void makeAcyclic();
	
	/* The minimum distance between two nodes.
	*/
	QSize grid() const;
	void setGrid(QSize);
	
	/* Enable/Disable debug mode
	*/
	void setDebugMode(bool w);
	
signals:
	/* Emitted when a layout step has been finished. This allows the visualization
	 * of the layout process.
	*/
	void sigDebugStep(QString);
	
	/* Emitted when a new layout stage has been reached.
	*/
	void sigDebugStage(QString name);
	
private:
	void initVars();
	void destroy();
	void layerAssignment(CGraph& G, NodeTable& nt);
	void layerAssignment_priv(Node parent, NodeTable& nt, int row);
	int layoutMaxX(std::vector<Node>& col);
	
protected:
	Graph oGraph;
	bool oAutoDelete;
	bool oDebug;
	QSize oGrid;
};

#endif
