/*-*-c++-*-
 * cline.cpp
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#include "cline.h"

#include <stdlib.h>

CLine::CLine()
{
	oArrow = true;
	oReversed = false;
	oEdge = 0;
}

CLine::~CLine()
{
}

QPoint
CLine::p1() const
{
	return oP1;
}

QPoint
CLine::p2() const
{
	return oP2;
}

void
CLine::setP1(const QPoint& p)
{
	oP1 = p;
}

void
CLine::setP2(const QPoint& p)
{
	oP2 = p;
}

bool
CLine::drawArrow() const
{
	return oArrow;
}

void
CLine::setDrawArrow(bool w)
{
	oArrow = w;
}

bool
CLine::reversed() const
{
	return oReversed;
}

void
CLine::setReversed(bool w)
{
	oReversed = w;
}

QString
CLine::label() const
{
	return oLabel;
}

void
CLine::setLabel(const QString& label)
{
	oLabel = label;
}

Edge
CLine::edge() const
{
	return oEdge;
}

void
CLine::setEdge(Edge e)
{
	oEdge = e;
}

int
CLine::y() const
{
	return (oP1.y() < oP2.y()) ? oP1.y() : oP2.y();
}

int
CLine::height() const
{
	return abs(oP1.y() - oP2.y());
}

int
CLine::x() const
{
	return (oP1.x() < oP2.x()) ? oP1.x() : oP2.x();
}

int
CLine::width() const
{
	return abs(oP1.x() - oP2.x());
}

bool operator<(const CLine& x, const CLine& y)
{
	return x.x() < y.x();
}

bool operator<=(const CLine& x, const CLine& y)
{
	return x.x() <= y.x();
}

bool operator==(const CLine& x, const CLine& y)
{
	return x.x() == y.x();
}

bool operator>=(const CLine& x, const CLine& y)
{
	return x.x() >= y.x();
}

bool operator>(const CLine& x, const CLine& y)
{
	return x.x() > y.x();
}
