/*-*-c++-*-
 * cline.h
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#ifndef CLINE_H
#define CLINE_H

#include <qpoint.h>
#include <qstring.h>

class CEdge;
typedef CEdge* Edge;

/**This is a line belonging to an edge. With these lines it is possible
  *to create bended edges without the need to create virtual nodes.
  *@author Michael Geselbracht
  */

class CLine {
public: 
	CLine();
	~CLine();
	
	QPoint p1() const;
	QPoint p2() const;
	void setP1(const QPoint& p);
	void setP2(const QPoint& p);
	
	/* Return the y-coordinate of the line's rectangular.
	*/
	int y() const;
	
	int height() const;
	
	/* Return the x-coordinate of the line's rectangular.
	*/
	int x() const;
	
	int width() const;
	
	bool drawArrow() const;
	void setDrawArrow(bool w);
	
	bool reversed() const;
	void setReversed(bool w);
	
	QString label() const;
	void setLabel(const QString&);
	
	Edge edge() const;
	void setEdge(Edge e);
	
private:
	QPoint oP1;
	QPoint oP2;
	bool oArrow;
	bool oReversed;
	QString oLabel;
	Edge oEdge;
};

bool operator<(const CLine& x, const CLine& y);
bool operator<=(const CLine& x, const CLine& y);
bool operator==(const CLine& x, const CLine& y);
bool operator>(const CLine& x, const CLine& y);
bool operator>=(const CLine& x, const CLine& y);

#endif
