/*-*-c++-*-
 * clinegroup.cpp
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#include "clinegroup.h"
#include "cnode.h"
#include "cedge.h"

#include <algorithm>
#include <limits.h>

CLineGroup::CLineGroup()
{
	oMarked = false;
}

CLineGroup::~CLineGroup()
{
}

CLine&
CLineGroup::insert(const CLine& x)
{
	push_back(x);
	CLine& last = back();
	sort();
	
	return last;
}

void
CLineGroup::sort()
{
	std::sort(begin(), end());
}

bool
CLineGroup::marked() const
{
	return oMarked;
}

void
CLineGroup::setMarked(bool w)
{
	oMarked = w;
}

int
CLineGroup::x() const
{
	int x = INT_MAX;
	
	for (CLineGroup::const_iterator it = begin(); it != end(); ++it) {
		if ((*it).x() < x) {
			x = (*it).x();
		}
	}
	
	return x;
}

int
CLineGroup::y() const
{
	return front().y();
}

int
CLineGroup::width() const
{
	QPoint w(0, 0);
	
	for (CLineGroup::const_iterator it = begin(); it != end(); ++it) {
		w += (*it).p1() - (*it).p2();
	}
	
	return abs(w.x());
}

int
CLineGroup::height() const
{
	QPoint h(0, 0);
	
	for (CLineGroup::const_iterator it = begin(); it != end(); ++it) {
		h += (*it).p1() - (*it).p2();
	}
	
	return abs(h.y());
}

Node
CLineGroup::from() const
{
	return (*this)[0].edge()->from();
}

Node
CLineGroup::to() const
{
	return (*this)[size() - 1].edge()->to();
}

bool operator<(const CLineGroup& x, const CLineGroup& y)
{
	return x[0].y() < y[0].y();
}

bool operator<=(const CLineGroup& x, const CLineGroup& y)
{
	return x[0].y() <= y[0].y();
}

bool operator==(const CLineGroup& x, const CLineGroup& y)
{
	return x[0].y() == y[0].y();
}
