/*-*-c++-*-
 * cngraphobject.h
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#ifndef CNGRAPHOBJECT_H
#define CNGRAPHOBJECT_H

#include <qframe.h>
#include <qstring.h>

class CGraphHeader;
class CNode;
typedef CNode* Node;

/**
  *@author Michael Geselbracht
  */

class CNGraphObject : public QFrame {
   Q_OBJECT

public:
	CNGraphObject(const QString& label, QWidget* parent=0, const char* name=0);
	~CNGraphObject();
	
	QWidget* widget();
	void setWidget(QWidget* child);
	void replaceWidget(QWidget* child, bool keepSize=false);
	
	Node node() const;
	void setNode(Node);
	
	bool marked() const;
	void setMarked(bool w);
	
	bool isViewMany() const;
	Node oneChild() const;
	void setOneChild(Node);
	
	inline bool isMinimized() const {
		return oOldSize.isValid();
	}
	
	/* Grab mouse events from header widget.
	*/
	virtual bool eventFilter(QObject* obj, QEvent* e);
	
	virtual void setCursor(const QCursor&);
	virtual void unsetCursor();

signals:
	void sigClose(CNGraphObject*);
	
	/* Emitted after toggleMinimize()
	*/
	void sigResized(CNGraphObject*);

private slots:
	void slotClose();
	void slotMinimize();
	
protected:
	virtual void resizeEvent(QResizeEvent*);
	void initAttributes();
	void toggleMinimize();
	
private:
	QWidget* oWidget;
	CGraphHeader* oLabel;
	bool oMarked;
	
	/* whoami
	*/
	Node oNode;
	
	Node oOneChild;
	
	/* The old size just before minimizing
	*/
	QSize oOldSize;
};

#endif
