/*-*-c++-*-
 * cnode.cpp
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#include "cnode.h"
#include "cedge.h"
#include "cedgeset.h"

#include <qwidget.h>

CNode::CNode()
{
}

CNode::CNode(const CNode& node)
{
	setNode(node);
}

CNode&
CNode::operator=(const CNode& node)
{
	setNode(node);
	
	return *this;
}

CNode::~CNode()
{
}

void
CNode::setNode(const CNode& node)
{
	oData = node.oData;
	oData.shadow = (CNode*) &node;
	oData.edges.clear();
}

int
CNode::x() const
{
	return oData.x;
}

int
CNode::y() const
{
	return oData.y;
}

void
CNode::setX(int x)
{
	oData.x = (x > 1) ? x : 1;
}

void
CNode::setY(int y)
{
	oData.y = (y > 1) ? y : 1;
}

int
CNode::width() const
{
	return (oData.widget == 0) ? oData.width : oData.widget->width();
}

int
CNode::height() const
{
	return (oData.widget == 0) ? oData.height : oData.widget->height();
}

void
CNode::setWidth(int w)
{
	oData.width = w;
	if (oData.widget != 0) {
		oData.widget->resize(w, oData.widget->height());
	}
}

void
CNode::setHeight(int h)
{
	oData.height = h;
	if (oData.widget != 0) {
		oData.widget->resize(oData.widget->width(), h);
	}
}

QWidget*
CNode::widget()
{
	return oData.widget;
}

void
CNode::setWidget(QWidget* w)
{
	if (oData.widget != 0 && w == 0) { // save dimensions
		oData.x = oData.widget->x();
		oData.x = oData.widget->y();
		oData.width = oData.widget->width();
		oData.height = oData.widget->height();
	}
	
	oData.widget = w;
}

void
CNode::addEdge(Edge e)
{
	oData.edges.add(e);
}

void
CNode::removeEdge(Edge e)
{
	oData.edges.remove(e);
}

EdgeSet
CNode::edges()
{
	return &(oData.edges);
}

int
CNode::degree() const
{
	return oData.edges.count();
}

int
CNode::indeg() const
{
	int count = 0;
	Edge e;
	
	forall_edges(e, (EdgeSet) &(oData.edges)) {
		if (e->to() == this) {
			count++;
		}
	}
	
	return count;
}

int
CNode::outdeg() const
{
	int count = 0;
	Edge e;
	
	forall_edges(e, (EdgeSet) &(oData.edges)) {
		if (e->from() == this) {
			count++;
		}
	}
	
	return count;
}

bool
CNode::visited() const
{
	return oData.visited;
}

void
CNode::setVisited(bool w)
{
	oData.visited = w;
}

int
CNode::value() const
{
	return oData.value;
}

void
CNode::setValue(int v)
{
	oData.value = v;
}

Node
CNode::shadow()
{
	return oData.shadow;
}

void
CNode::setShadow(Node s)
{
	oData.shadow = s;
}

bool
CNode::virtualNode() const
{
	return oData.isvirtual;
}

void
CNode::setVirtualNode(bool w)
{
	oData.isvirtual = w;
	oData.height = 10;
}

QString
CNode::label() const
{
	if (oData.widget != 0) {
		return oData.widget->caption();
	}
	
	return oData.label;
}

void
CNode::setLabel(const QString& lbl)
{
	if (oData.widget != 0) {
		oData.widget->setCaption(lbl);
	}
	else {
		oData.label = lbl;
	}
}

Node
CNode::next()
{
	Node v = 0;
	Edge e;
	forall_edges(e, &oData.edges) {
		if (e->from() == this) {
			v = e->to();
			break;
		}
	}
	
	return v;
}

Node
CNode::prev()
{
	Node v = 0;
	Edge e;
	forall_edges(e, &oData.edges) {
		if (e->to() == this) {
			v = e->from();
			break;
		}
	}
	
	return v;
}

