/*-*-c++-*-
 * cnodeset.cpp
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#include "cnodeset.h"

CNodeSet::CNodeSet()
{
	oNodeSet = new NodeSet_t;
	oNodeSet->count = 1;
}

CNodeSet::CNodeSet(const CNodeSet& ns)
{
	oNodeSet = 0;
	setNodes(ns);
}

CNodeSet::~CNodeSet()
{
	destroy();
}

CNodeSet&
CNodeSet::operator=(const CNodeSet& ns)
{
	setNodes(ns);
	
	return *this;
}

void
CNodeSet::destroy()
{
	if (oNodeSet != 0) {
		oNodeSet->count--;
		
		if (oNodeSet->count == 0) {
			delete oNodeSet;
			oNodeSet = 0;
		}
	}
}

void
CNodeSet::setNodes(const CNodeSet& ns)
{
}

std::list<Node>::iterator
CNodeSet::begin()
{
	return oNodeSet->nodes.begin();
}

std::list<Node>::iterator
CNodeSet::end()
{
	return oNodeSet->nodes.end();
}

void
CNodeSet::add(Node v)
{
	oNodeSet->nodes.push_back(v);
}

void
CNodeSet::add(NodeSet ns)
{
	Node v;
	forall_nodes(v, ns) {
		oNodeSet->nodes.push_back(v);
	}
}

void
CNodeSet::remove(Node v)
{
	Iterator it = oNodeSet->nodes.begin();
	while (it != oNodeSet->nodes.end()) {
		if (*it == v) {
			oNodeSet->nodes.erase(it);
			break;
		}
		
		++it;
	}
}

void
CNodeSet::clear()
{
	destroy();
	
	oNodeSet = new NodeSet_t;
	oNodeSet->count = 1;
}

int
CNodeSet::count() const
{
	return oNodeSet->nodes.size();
}

bool
CNodeSet::empty() const
{
	return count() == 0;
}
