/*-*-c++-*-
 * $Id: cparseview.cpp 1079 2004-10-22 19:59:34Z troyhebe $
 *
 * This file is part of qlcrash, a GUI for the dump-analysis tool lcrash.
 *
 * Copyright (C) 2001 IBM Deutschland Entwicklung GmbH, IBM Corporation
 *
 * Authors:
 * Michael Geselbracht (let@users.sourceforge.net)
 * Fritz Elfert (elfert@de.ibm.com)
 * Michael Holzheu (holzheu@de.ibm.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */

#include "cparseview.h"
#include "parser.h"

#include <qpushbutton.h>
#include <qlistview.h>
#include <qlabel.h>
#include <qheader.h>

CParseView::CParseView(CParser* parser, QWidget* parent) : QDialog(parent, 0, true)
{
    oParser = parser;
    oMessage = new QLabel(tr(
        "<p><b>A parse error has occured.</b></p>"
        "<p>This might be a bug in qlcrash or just an incompatibility"
        "between the lcrash and qlcrash versions.</p>"
    ), this);
    oMessage->setAlignment(AlignLeft|AlignTop);

    oClose = new QPushButton(tr("Close"), this);
    oClose->setAutoResize(true);
    oShowDetails = new QPushButton(tr("Details") + "...", this);
    oShowDetails->setAutoResize(true);
    oDebugList = 0;

    connect(oClose, SIGNAL(clicked()), SLOT(slotClose()));
    connect(oShowDetails, SIGNAL(clicked()), SLOT(slotShowDetails()));

    setMinimumSize(300, 240);
}

CParseView::~CParseView()
{
}

void
CParseView::resizeEvent(QResizeEvent*)
{
    if (oDebugList == 0) { // normal text message
        oMessage->setGeometry(5, 5, width() - 10, 75);
        oShowDetails->move(width() - oShowDetails->width() - 5, oMessage->y() + oMessage->height() + 5);
    }
    else { // debug view
        oMessage->setGeometry(5, 5, width() - 10, 25);
        oDebugList->setGeometry(
            5,
            oMessage->y() + oMessage->height() + 5,
            width() - 10,
            height() - oMessage->height() - oMessage->y() - oClose->height() - 20
        );
    }

    oClose->move(width()/2 - oClose->width()/2, height() - oClose->height() - 5);
}

void
CParseView::slotShowDetails()
{
    delete oShowDetails;
    oDebugList = new QListView(this);
    oDebugList->show();
    oDebugList->addColumn("");
    oDebugList->header()->hide();
    oDebugList->setSorting(-1);
    oDebugList->setFont(font());
    oMessage->setText(tr("Parser messages") + ":");

    // fill list
    QStringList::ConstIterator it = oParser->debugString().begin();
    QListViewItem* after = 0;
    while (it != oParser->debugString().end()) {
        QListViewItem* item = (after == 0)
            ? new QListViewItem(oDebugList)
            : new QListViewItem(oDebugList, after)
        ;
        after = item;
        item->setText(0, *it);
        ++it;
    }
    oDebugList->ensureItemVisible(after);
    oDebugList->setSelected(after, true);

    resizeEvent(0);
}

void
CParseView::slotClose()
{
    accept();
}

