/*-*-c++-*-
 * cpreferencesfontimpl.cpp
 * Copyright (c) 2002 Michael Geselbracht <let@user.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version. See the file COPYING for more
 * information.
 */


#include "cpreferencesfontimpl.h"
#include "qlcrashdoc.h"
#include "cconfigmanager.h"


#include <qlabel.h>
#include <qcheckbox.h>
#include <qfontdialog.h>
#include <qpushbutton.h>

CPreferencesFontImpl::CPreferencesFontImpl(QLcrashDoc* doc, QWidget* parent) : CPreferencesFont(parent)
{
    oDoc = doc;
    init();
}

CPreferencesFontImpl::~CPreferencesFontImpl()
{
}

void
CPreferencesFontImpl::init()
{
    QString family;
    int size;
    bool useFixed;
    QFont font;

    family = oDoc->configManager()->item(CFG_GRP_APP, CFG_ATT_APP_FONTFAMILY, DEF_APP_FONTFAMILY);
    size = oDoc->configManager()->item(CFG_GRP_APP, CFG_ATT_APP_FONTSIZE, DEF_APP_FONTSIZE);
    font.setFamily(family);
    font.setPointSize(size);
    oGlobalFont->setFont(font);
    oGlobalFont->setText(font.family() + " " + QString::number(font.pointSize()) + "p");

    family = oDoc->configManager()->item(CFG_GRP_APP, CFG_ATT_APP_FIXEDFONTFAMILY, DEF_APP_FIXEDFONTFAMILY);
    size = oDoc->configManager()->item(CFG_GRP_APP, CFG_ATT_APP_FIXEDFONTSIZE, DEF_APP_FIXEDFONTSIZE);
    font.setFamily(family);
    font.setPointSize(size);
    font.setFamily(family);
    font.setPointSize(size);
    oFixedFont->setFont(font);
    oFixedFont->setText(font.family() + " " + QString::number(font.pointSize()) + "p");

    family = oDoc->configManager()->item(CFG_GRP_CONSOLE, CFG_ATT_FONTFAMILY, DEF_APP_FIXEDFONTFAMILY);
    size = oDoc->configManager()->item(CFG_GRP_CONSOLE, CFG_ATT_FONTSIZE, DEF_APP_FIXEDFONTSIZE);
    font.setFamily(family);
    font.setPointSize(size);
    font.setFamily(family);
    font.setPointSize(size);
    oConsoleFont->setFont(font);
    oConsoleFont->setText(font.family() + " " + QString::number(font.pointSize()) + "p");
    useFixed = oDoc->configManager()->item(CFG_GRP_CONSOLE, CFG_ATT_USEFIXEDFONT, (int)true);
    oConsoleUseFixed->setChecked(useFixed);

    family = oDoc->configManager()->item(CFG_GRP_DISASSEMBLE, CFG_ATT_FONTFAMILY, DEF_DISASSEMBLE_FONTFAMILY);
    size = oDoc->configManager()->item(CFG_GRP_DISASSEMBLE, CFG_ATT_FONTSIZE, DEF_DISASSEMBLE_FONTSIZE);
    font.setFamily(family);
    font.setPointSize(size);
    font.setFamily(family);
    font.setPointSize(size);
    oDisassembleFont->setFont(font);
    oDisassembleFont->setText(font.family() + " " + QString::number(font.pointSize()) + "p");
    useFixed = oDoc->configManager()->item(CFG_GRP_DISASSEMBLE, CFG_ATT_USEFIXEDFONT, (int)true);
    oDisassembleUseFixed->setChecked(useFixed);

    family = oDoc->configManager()->item(CFG_GRP_DUMP, CFG_ATT_FONTFAMILY, DEF_DUMP_FONTFAMILY);
    size = oDoc->configManager()->item(CFG_GRP_DUMP, CFG_ATT_FONTSIZE, DEF_DUMP_FONTSIZE);
    font.setFamily(family);
    font.setPointSize(size);
    font.setFamily(family);
    font.setPointSize(size);
    oDumpFont->setFont(font);
    oDumpFont->setText(font.family() + " " + QString::number(font.pointSize()) + "p");
    useFixed = oDoc->configManager()->item(CFG_GRP_DUMP, CFG_ATT_USEFIXEDFONT, (int)true);
    oDumpUseFixed->setChecked(useFixed);

    slotConsoleUseFixed();
    slotDisassembleUseFixed();
    slotDumpUseFixed();
}

void
CPreferencesFontImpl::slotSelectGlobal()
{
    bool ret;
    QFont f = QFontDialog::getFont(&ret, oGlobalFont->font(), this);
    if (ret) {
        oGlobalFont->setFont(f);
        oGlobalFont->setText(f.family() + " " + QString::number(f.pointSize()) + "p");
    }
}

void
CPreferencesFontImpl::slotSelectFixed()
{
    bool ret;
    QFont f = QFontDialog::getFont(&ret, oFixedFont->font(), this);
    if (ret) {
        oFixedFont->setFont(f);
        oFixedFont->setText(f.family() + " " + QString::number(f.pointSize()) + "p");
    }
}

void
CPreferencesFontImpl::slotSelectDisassemble()
{
    bool ret;
    QFont f = QFontDialog::getFont(&ret, oDisassembleFont->font(), this);
    if (ret) {
        oDisassembleFont->setFont(f);
        oDisassembleFont->setText(f.family() + " " + QString::number(f.pointSize()) + "p");
    }
}

void
CPreferencesFontImpl::slotSelectConsole()
{
    bool ret;
    QFont f = QFontDialog::getFont(&ret, oConsoleFont->font(), this);
    if (ret) {
        oConsoleFont->setFont(f);
        oConsoleFont->setText(f.family() + " " + QString::number(f.pointSize()) + "p");
    }
}

void
CPreferencesFontImpl::slotSelectDump()
{
    bool ret;
    QFont f = QFontDialog::getFont(&ret, oDumpFont->font(), this);
    if (ret) {
        oDumpFont->setFont(f);
        oDumpFont->setText(f.family() + " " + QString::number(f.pointSize()) + "p");
    }
}

void
CPreferencesFontImpl::slotConsoleUseFixed()
{
    oConsoleFont->setEnabled(!oConsoleUseFixed->isOn());
    oConsoleSelect->setEnabled(!oConsoleUseFixed->isOn());
}

void
CPreferencesFontImpl::slotDisassembleUseFixed()
{
    oDisassembleFont->setEnabled(!oDisassembleUseFixed->isOn());
    oDisassembleSelect->setEnabled(!oDisassembleUseFixed->isOn());
}

void
CPreferencesFontImpl::slotDumpUseFixed()
{
    oDumpFont->setEnabled(!oDumpUseFixed->isOn());
    oDumpSelect->setEnabled(!oDumpUseFixed->isOn());
}

void
CPreferencesFontImpl::saveConfig()
{
    oDoc->configManager()->setItem(CFG_GRP_APP, CFG_ATT_FONTFAMILY, oGlobalFont->font().family());
    oDoc->configManager()->setItem(CFG_GRP_APP, CFG_ATT_FONTSIZE, oGlobalFont->font().pointSize());
    oDoc->configManager()->setItem(CFG_GRP_APP, CFG_ATT_APP_FIXEDFONTFAMILY, oFixedFont->font().family());
    oDoc->configManager()->setItem(CFG_GRP_APP, CFG_ATT_APP_FIXEDFONTSIZE, oFixedFont->font().pointSize());
    oDoc->configManager()->setItem(CFG_GRP_CONSOLE, CFG_ATT_FONTFAMILY, oConsoleFont->font().family());
    oDoc->configManager()->setItem(CFG_GRP_CONSOLE, CFG_ATT_FONTSIZE, oConsoleFont->font().pointSize());
    oDoc->configManager()->setItem(CFG_GRP_CONSOLE, CFG_ATT_USEFIXEDFONT, oConsoleUseFixed->isOn());
    oDoc->configManager()->setItem(CFG_GRP_DISASSEMBLE, CFG_ATT_FONTFAMILY, oDisassembleFont->font().family());
    oDoc->configManager()->setItem(CFG_GRP_DISASSEMBLE, CFG_ATT_FONTSIZE, oDisassembleFont->font().pointSize());
    oDoc->configManager()->setItem(CFG_GRP_DISASSEMBLE, CFG_ATT_USEFIXEDFONT, oDisassembleUseFixed->isOn());
    oDoc->configManager()->setItem(CFG_GRP_DUMP, CFG_ATT_FONTFAMILY, oDumpFont->font().family());
    oDoc->configManager()->setItem(CFG_GRP_DUMP, CFG_ATT_FONTSIZE, oDumpFont->font().pointSize());
    oDoc->configManager()->setItem(CFG_GRP_DUMP, CFG_ATT_USEFIXEDFONT, oDumpUseFixed->isOn());
}
