/***************************************************************************
                                 graph.h
                             -------------------
    begin                : Wed Oct 31 2001
    copyright            : (C) 2001 by Michael Geselbracht
    email                : let@uni.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPH_H
#define GRAPH_H

#include "cnodeset.h"
#include "cedgeset.h"

#include <vector>
#include <set>

class CGraph;
class CLineGroup;

typedef CGraph* Graph;
typedef std::vector<Node> NodeVector;
typedef std::vector<NodeVector> NodeTable;
typedef std::multiset<CLineGroup> CLineSet;
typedef CLineSet* LineSet;

/**This class represents a directed graph
  *@author Michael Geselbracht
  */

class CGraph {
public: 
	CGraph();
	CGraph(const CGraph&); // deep copy
	~CGraph();
	
	void destroy();
	
	CGraph& operator=(const CGraph& g);
	
	/* Make a deep copy from an existing graph.
	*/
	void setGraph(const CGraph& g);
	
	/* Return all nodes of this graph.
	*/
	NodeSet nodes();
	
	/* Return all edges of this graph.
	*/
	EdgeSet edges();
	
	/* Return all lines of this graph.
	*/
	LineSet lines();
	
	/* Create a new node and add it to the graph.
	*/
	Node createNode();
	
	/* Create a new edge and add it to the graph.
	*/
	Edge createEdge(Node from, Node to);
	
	/* Remove an edge from this graph.
	*/
	void removeEdge(Edge e);
	
	/* Remove egde from parent to child.
	*/
	void removeEdge(Node parent, Node child);
	
	/* Remove a node from this graph.
	*/
	void removeNode(Node v);
	
	/* Check if this graph is acyclic.
	*/
	bool isAcyclic();
	
	/* Return an arbitrary sink of this graph.
	*/
	Node sink();
	
	/* Return an arbitrary source of this graph.
	*/
	Node source();
	
	/* Print debug output.
	*/
	void debug();
	
	/* Shall the layout object modify vertex coordinates ?
	*/
	bool fixedPos() const;
	void setFixedPos(bool w);
	
private:
	NodeSet oNodes;
	EdgeSet oEdges;
	LineSet oLines;
	bool oFixedPos;
	static int sID;
};

#endif
